# -*- coding: utf-8 -*-
#
# Copyright (C) 2022-2026 CERN.
# Copyright (C) 2022-2023 TU Wien.
# Copyright (C) 2024-2026 Graz University of Technology.
#
# Invenio-Users-Resources is free software; you can redistribute it and/or
# modify it under the terms of the MIT License; see LICENSE file for more
# details.

"""Invenio module providing management APIs for users and roles/groups."""

__version__ = "10.4.1"

__all__ = ("__version__",)
